/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
#include "oapr/buildconfig.hpp"
#include "oapr/condition.hpp"
#include "oapr/exception.hpp"
#include "oapr/mutex.hpp"


FILE_VERSION("$Revision: 1.6.2.1 $",oapr_cond_cpp);

namespace oapr
{
  condition::condition()
    : m_cond(0)
  {
  }

  condition::~condition()
  {
    if(m_cond != 0){
      PR_DestroyCondVar(m_cond);
    }
  }

  void condition::wait(mutex& mtx)
  {
    if(m_cond != 0){
      OAPR_THROW(invalid_state,"condition is already initialized");
    }
    m_cond = PR_NewCondVar(mtx.m_mutex);
    if(m_cond == 0){
      OAPR_THROW(runtime_error,"failed to create condition");
    }

    oapr_status_t wt = PR_WaitCondVar(m_cond,PR_INTERVAL_NO_TIMEOUT);
    if(wt != OAPR_SUCCESS){
      OAPR_THROW(runtime_error,"failed wait on condition");
    }
  }

  oapr_status_t condition::wait(mutex& mtx, const oapr_time_interval_t& timeout)
  {
    if(m_cond){
      OAPR_THROW(invalid_state, "condition is already initialized");
    }
    oapr_status_t rc = PR_WaitCondVar(m_cond,timeout);
    return rc;
    /* need to redo this logic for nspr
    switch(rc){
    case OAPR_SUCCESS: return rc;
    case : return OAPR_TIMEOUT;
    default: OAPR_THROW_OAPR(OAPR_EINTR);
    };
    */
  }

  void condition::signal()
  {
    if(m_cond){
      OAPR_CHECK(PR_NotifyCondVar(m_cond));
    }
  }
};
