/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

#ifndef OAPR_ERRNO_HPP
#define OAPR_ERRNO_HPP


/**
 * OAPR reserves an additional section of oapr_status_t codes.
 */
 
#define OAPR_ERRSPACE_SIZE 10000
  // Size of OAPR error space

#define OAPR_START_ERROR 100
  // Start of OAPR error space

/**
 * 40000 codes reserved for user-defined error/status codes, starting from OAPR_START_USERERR.
 */

#define OAPR_USER_ERRSPACE_SIZE (OAPR_ERRSPACE_SIZE-OAPR_START_ERROR)
  // Size of user error space

#define OAPR_START_USERERR   5000
  // Start of user error space
  
/**
 * Additional status codes
 */
/**
 * @defgroup OAPR_Error Additional Error Codes
 * <PRE>
 * <b>ADDITIONAL OAPR ERROR VALUES</b>
 *
 * OAPR_EUNCAUGHT_EXCEPTION          An uncaught exception has caused the thread to terminate.
 * OAPR_INTERNAL_THREAD_LOGIC_ERROR  There was an unexpected error in the threading logic. Should never happen, contact OAPR authors.
 * </PRE>
 */

#define OAPR_EUNCAUGHT_EXCEPTION          (OAPR_START_ERROR + 1)
#define OAPR_INTERNAL_THREAD_LOGIC_ERROR  (OAPR_START_ERROR + 2)
#define OAPR_DECIMAL_RANGE_ERROR          (OAPR_START_ERROR + 3) // Converting string representation to decimal, underflow or overflow error occurred.
#define OAPR_POWER_EXPONENT_NOT_POSITIVE  (OAPR_START_ERROR + 4)

/**
 * @defgroup APR_STATUS_IS Status Value Tests
 * @warning For any particular error condition, more than one of these tests
 *      may match. This is because platform-specific error codes may not
 *      always match the semantics of the POSIX codes these tests (and the
 *      correcponding APR error codes) are named after. A notable example
 *      are the APR_STATUS_IS_ENOENT and APR_STATUS_IS_ENOTDIR tests on
 *      Win32 platforms. The programmer should always be aware of this and
 *      adjust the order of the tests accordingly.
 * @{
 */

/** An uncaught exception has caused the thread to terminate */
#define OAPR_STATUS_IS_EUNCAUGHT_EXCEPTION(s)    ((s) == OAPR_EUNCAUGHT_EXCEPTION)
/** @} */

 
#endif
