/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIA* state() == tsTerminated.BILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY exceptionOUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#ifndef OAPR_EXC_LOGIC_HPP
#define OAPR_EXC_LOGIC_HPP
#include "oapr/buildconfig.hpp"
#include "oapr/exc_base.hpp"

FILE_VERSION("$Revision: 1.1 $",oapr_exc_logic_hpp);

namespace oapr
{
  class logic_error : public exception_base
  {
  public:
    logic_error(int line,
                std::string method,
                std::string file,
                std::string msg,
                int level = 0)
      : exception_base(line,method,file,msg,level){}
    logic_error(const logic_error& le,
                int line,
                std::string method,
                std::string file,
                std::string src,
                std::string msg,
                int level = 0)
      : exception_base(le,line,method,file,src,msg,level){}
    virtual ~logic_error() throw(){}
  };
  class domain_error : public logic_error
  {
  public:
    domain_error(int line,
		std::string method,
		std::string file,
		std::string msg,
		int level = 0)
      : logic_error(line,method,file,msg,level){}
    domain_error(const domain_error& de,
		 int line,
		std::string method,
		std::string file,
		std::string src,
		std::string msg,
		int level = 0)
      : logic_error(de,line,method,file,src,msg,level){}
    virtual ~domain_error() throw(){}
  };
  class invalid_arg : public logic_error
  {
  public:
    invalid_arg(int line,
		std::string method,
		std::string file,
		std::string msg,
		int level = 0)
      : logic_error(line,method,file,msg,level){}
    invalid_arg(const invalid_arg& ia,
		int line,
		std::string method,
		std::string file,
		std::string src,
		std::string msg,
		int level = 0)
      : logic_error(ia,line,method,file,src,msg,level){}
    virtual ~invalid_arg() throw(){}
  };
  class length_error : public logic_error
  {
  public:
    length_error(int line,
		 std::string method,
		 std::string file,
		 std::string msg,
		 int level = 0)
      : logic_error(line,method,file,msg,level){}
    length_error(const length_error& le,
		 int line,
		 std::string method,
		 std::string file,
		 std::string src,
		 std::string msg,
		 int level = 0)
      : logic_error(le,line,method,file,src,msg,level){}
    virtual ~length_error() throw(){}
  };
  class out_of_range : public logic_error
  {
  public:
    out_of_range(int line,
		 std::string method,
		 std::string file,
		 std::string msg,
		 int level = 0)
      : logic_error(line,method,file,msg,level){}
    out_of_range(const out_of_range& oor,
		 int line,
		 std::string method,
		 std::string file,
		 std::string src,
		 std::string msg,
		 int level = 0)
      : logic_error(oor,line,method,file,src,msg,level){}
    virtual ~out_of_range() throw(){}
  };

};

#endif
