/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#ifndef OAPR_EXCEPTION_HPP
#define OAPR_EXCEPTION_HPP
#include "oapr/oapr.hpp"
#include "oapr/buildconfig.hpp"
#include "oapr/exc_base.hpp"
#include "oapr/exc_logic.hpp"
#include "oapr/exc_runtime.hpp"
#include "oapr/exc_oapr.hpp"

FILE_VERSION("$Revision: 1.5 $",oapr_exception_hpp);

#ifdef WIN32
#define OAPR_FUNCTION __FUNCTION__
#else
#define OAPR_FUNCTION __PRETTY_FUNCTION__
#endif

/** @warning when throwing oapr_exception, msg is a status code not the usual string type. */

#define OAPR_THROW(exc,msg) throw exc(__LINE__, \
						OAPR_FUNCTION, \
						__FILE__, \
						msg);

#define OAPR_THROW_LVL(exc,msg,lvl) throw exc(__LINE__, \
							OAPR_FUNCTION, \
							__FILE__, \
							msg, \
							lvl);
#define OAPR_RETHROW(exc,caught) throw exc(caught, \
						 __LINE__, \
						 OAPR_FUNCTION, \
						 __FILE__, \
						 caught.source(), \
						 caught.what(), \
						 caught.level());
#define OAPR_THROW_OAPR(code) throw oapr_exception(__LINE__, OAPR_FUNCTION, __FILE__, code);

#define OAPR_CATCH() \
catch(const oapr::oapr_exception& oe){ \
  OAPR_RETHROW(oapr::oapr_exception,oe); \
}catch(const oapr::exception_base& eb){ \
  OAPR_RETHROW(oapr::exception_base,eb); \
}catch(const std::exception& e){ \
  OAPR_THROW(oapr::exception_base,e.what()); \
}catch(...){ \
  OAPR_THROW(oapr::exception_base,"Unknown exception"); \
} 

/**\todo Level generated automatically from oapr_status_t code?*/
#define OAPR_CHECK(code) oapr::oapr_exception::check_rc(__LINE__, \
                                   OAPR_FUNCTION, \
                                   __FILE__, \
                                   code);
#define OAPR_CHECK_S(code,src) oapr::oapr_exception::check_rc(__LINE__, \
                                         OAPR_FUNCTION, \
                                         __FILE__, \
                                         code);
#define OAPR_CHECK_MSG(code,msg) oapr::oapr_exception::check_rc(__LINE__, \
                                           OAPR_FUNCTION, \
                                           __FILE__, \
                                           code, \
                                           msg);
#define OAPR_CHECK_LVL(code,lvl) oapr::oapr_exception::check_rc(__LINE__, \
                                           OAPR_FUNCTION, \
                                           __FILE__, \
                                           code, \
                                           lvl);
#define OAPR_CHECK_ML(code,msg,lvl) oapr::oapr_exception::check_rc(__LINE__, \
                                           OAPR_FUNCTION, \
                                           __FILE__, \
                                           code, \
                                           msg, \
                                           lvl);

/** Check for null on a function */
#define OAPR_CHECK_NULL(code, throwcode) { if(!code) throw oapr::oapr_exception(__LINE__, \
						OAPR_FUNCTION, \
						__FILE__, \
						throwcode); }
#define OAPR_CHECK_NONZERO(code, throwcode) { if(code) throw oapr::oapr_exception(__LINE__, \
						OAPR_FUNCTION, \
						__FILE__, \
						throwcode); }
#endif
