/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
#include "oapr-config.h"
#include "oapr/buildconfig.hpp"
#include "oapr/file.hpp"
#include "oapr/exception.hpp"
#include "oapr/exc_utils.hpp"


FILE_VERSION("$Revision: 1.4 $",file_cpp);

namespace oapr
{

  oapr_status_t basic_file::open(const std::string& addr, oapr_int32_t flags, oapr_int32_t perms)
  {
    if(is_open()){
      OAPR_THROW(logic_error,"already has an open file, close first");
    }

    m_fileName=addr;

    m_file = PR_Open(m_fileName.c_str(),flags,perms);    
    if(m_file){
      m_isOpen=true;
      return OAPR_SUCCESS;
    }else{
      return OAPR_FAILURE;
    }

  }

  oapr_status_t basic_file::close()
  {
    oapr_status_t rv = OAPR_SUCCESS;
    if(m_file){
      rv = PR_Close(m_file);
      if(rv == OAPR_SUCCESS){
	m_isOpen=false;
      }
    }
    return rv;
  }

  oapr_int32_t basic_file::write(const void* buff, oapr_int32_t toWrite)
  {
    if(!is_open()){
      OAPR_THROW(invalid_state,"file is not open");
    }
    oapr_int32_t rv = PR_Write(m_file,buff,toWrite);
    return rv;
  }
  oapr_int32_t basic_file::read(void* buff, oapr_int32_t toRead)const
  {
    if(!is_open()){
      OAPR_THROW(invalid_state,"file is not open");
    }
    oapr_int32_t rv = PR_Read(m_file,buff,toRead);
    return rv;
  }

  oapr_status_t basic_file::flush()
  {
    if(!is_open()){
      OAPR_THROW(invalid_state,"file is not open");
    }
    return PR_Sync(m_file);
  }


  oapr_status_t remove(const basic_file& f)
  {
    return PR_Delete(f.name().c_str());
  }
  oapr_status_t remove(const std::string &nm)
  {
    return PR_Delete(nm.c_str());
  }
  oapr_status_t rename(const basic_file& f,const std::string& to)
  {
    return PR_Rename(f.name().c_str(),to.c_str());
  }
  oapr_status_t rename(const std::string& from,const std::string& to)
  {
    return PR_Rename(from.c_str(),to.c_str());
  }
  oapr_status_t copy(const basic_file& f,const std::string& to)
  {
    return OAPR_NOT_IMPLEMENTED;
  }
  oapr_status_t copy(const std::string& from,const std::string& to)
  {
    return OAPR_NOT_IMPLEMENTED;
  }
  oapr_status_t append(const basic_file& f,const std::string& to)
  {
    return OAPR_NOT_IMPLEMENTED;
  }
  oapr_status_t append(const std::string& from,const std::string& to)
  {
    return OAPR_NOT_IMPLEMENTED;
  }

};
