/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
#ifndef OAPR_IO_FILE_HPP
#define OAPR_IO_FILE_HPP
#include "oapr/oapr.hpp"
#include "oapr/buildconfig.hpp"
#include "oapr/io.hpp"
#include <prio.h>

FILE_VERSION("$Revision: 1.6 $",file_hpp);

namespace oapr
{
  typedef PRFileDesc oapr_file_t;
  enum File_Perm {
    Read_Owner=00400,
    Read_Group=00040,
    Read_All=00004,
    Write_Owner=00200,
    Write_Group=00020,
    Write_All=00002,
    Exec_Owner=00100,
    Exec_Group=00010,
    Exec_All=00001
  };
  class LIBOAPR_API basic_file : public basic_io
  {
  public:
    basic_file()
      : basic_io()
	,m_file(0)
	,m_isOpen(false)
	,m_eof(false)
    {
    }
    basic_file(const basic_file& f)
      : basic_io(f),
	m_fileName(f.m_fileName),
	m_file(f.m_file),
	m_isOpen(f.m_isOpen),
	m_eof(f.m_eof)
    {
    }
    virtual ~basic_file()
    {
    }
    virtual std::string name()const{return m_fileName;}
    virtual oapr_status_t open(const std::string& addr,
		      oapr_int32_t flags,
		      oapr_int32_t perms);
    /**\todo operations we need
     * - duplicate
     * - setaside ?
     * - seek
     * - lock/unlock ?
     */

    /* generic buffer operations from the io class */
    virtual oapr_status_t close();
    virtual bool eof()const{return m_eof;}
    virtual bool is_open()const{return m_isOpen;}

    virtual oapr_int32_t write(const void* buff, oapr_int32_t toWrite);
    virtual oapr_int32_t read( void* buff,oapr_int32_t toRead)const;

    virtual oapr_status_t flush();
    virtual oapr_file_t* fd()const{return m_file;}

  protected:
    std::string m_fileName;
    oapr_file_t *m_file;
    bool m_isOpen;
    bool m_eof;
  };
  LIBOAPR_API oapr_status_t remove(const basic_file& f);
  LIBOAPR_API oapr_status_t remove(const std::string &nm);
  LIBOAPR_API oapr_status_t rename(const basic_file& f,const std::string& to);
  LIBOAPR_API oapr_status_t rename(const std::string& from,const std::string& to);
  LIBOAPR_API oapr_status_t copy(const basic_file& f,const std::string& to);
  LIBOAPR_API oapr_status_t copy(const std::string& from,const std::string& to);
  LIBOAPR_API oapr_status_t append(const basic_file& f,const std::string& to);
  LIBOAPR_API oapr_status_t append(const std::string& f,const std::string& to);

};
#endif
