/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#ifndef OAPR_IOFILESTREAM_HPP
#define OAPR_IOFILESTREAM_HPP
#include "oapr/oapr.hpp"
#include "oapr/buildconfig.hpp"
#include <iostream>
#include "boost/scoped_ptr.hpp"
#include "oapr/exception.hpp"
#include "oapr/exc_utils.hpp"
#include "oapr/filebuf.hpp"
FILE_VERSION("$Revision: 1.3 $",iofilestream_hpp);

namespace oapr
{
  template<typename charT,typename traits, std::streamsize bufszT=255>
  class basic_iofilestream : public std::basic_iostream<charT,traits>
  {
  public:
    //standard typedefs
    typedef charT                     char_type;
    typedef typename traits::int_type int_type;
    typedef typename traits::pos_type pos_type;
    typedef typename traits::off_type off_type;
    typedef traits                    traits_type;

    //non standard typdefs
    //typedef put stream buf type here
    typedef std::basic_ios<char_type,traits>           ios_type;
    typedef std::basic_iostream<char_type,traits>       iostream_type;
    //    typedef std::iostreambuf_iterator<char_type,traits> iostreambufiter_type;
    typedef basic_filebuf<char_type,traits_type>       filebuf_type;
    typedef basic_iofilestream<char_type,traits_type>   iofilestream_type;

  private:
    boost::scoped_ptr<filebuf_type> m_filebuf;

  public:
    basic_iofilestream()
      : iostream_type(NULL), m_filebuf(new filebuf_type)
    {
      this->init(m_filebuf.get());
    }
    basic_iofilestream(const char* nm, std::ios_base::openmode md,
		      oapr_int32_t perms)
      : iostream_type(NULL), m_filebuf(new filebuf_type)
    {
      this->init(m_filebuf.get());
      
    }
    virtual ~basic_iofilestream(){}

    bool is_open()const{return m_filebuf->is_open();}
    iofilestream_type* open(const char* nm, std::ios_base::openmode md,
			  oapr_int32_t perms=00200)
    {
      iofilestream_type* rv=0;
      if(m_filebuf->open(nm,md,perms)){
	rv =this;
      }
      return rv;
    }
    iofilestream_type* close()
    {
      iofilestream_type* rv=0;
      if(m_filebuf->close()){
	rv=this;
      }else{
	this->setstate(std::ios_base::failbit);
      }
      return rv;
    }
  };
};
#endif
