/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#ifndef OAPR_OSOCKSTREAM_HPP
#define OAPR_OSOCKSTREAM_HPP
#include "boost/scoped_ptr.hpp"
#include "oapr/sockbuf.hpp"
#include "oapr/exception.hpp"
#include "oapr/exc_utils.hpp"

namespace oapr
{
  /* following are general todo lists for this*/
  /** @todo handle the exception bits*/
  /** @todo handle the state bits*/
  template<typename charT, typename traits,
	   typename thread_pol_t=thread_pol_none,
	   std::streamsize buf_sz=255>
  class basic_osockstream : public std::basic_ostream<charT,traits>
  {
  public:
    //standard typedefs
    typedef charT                     char_type;
    typedef typename traits::int_type int_type;
    typedef typename traits::pos_type pos_type;
    typedef typename traits::off_type off_type;
    typedef traits                    traits_type;
    typedef thread_pol_t              thread_pol_type;


    //non standard typdefs
    //typedef put stream buf type here
    typedef std::basic_ios<char_type,traits>           ios_type;
    typedef std::basic_ostream<char_type,traits>       ostream_type;
    typedef std::ostreambuf_iterator<char_type,traits> ostreambufiter_type;
    typedef basic_sockbuf<char_type,traits_type,
			  thread_pol_type,buf_sz>      sockbuf_type;
    typedef basic_osockstream<char_type,traits_type,
			      thread_pol_type,buf_sz>  sockstream_type;

  private:
    sockinfo m_si;
    sockbuf_type* m_sockbuf;/**< the socket buffer */
    bool m_delbuf;
  public:
    basic_osockstream()
      : ostream_type(NULL)
	,m_si()
	,m_sockbuf(new sockbuf_type())
	,m_delbuf(true){
      try{
	this->init(m_sockbuf);
      }OAPR_CATCH();
    } 
    basic_osockstream(sockbuf_type* sb)
      : ostream_type(NULL)
	,m_si()
	,m_sockbuf(sb)
	,m_delbuf(false){
      this->init(m_sockbuf);
    }

    basic_osockstream(const char* host, unsigned short port,
		      int type, typename sockbuf_type::sockmodes md = sockbuf_type::client)
      : ostream_type(NULL)
	,m_si()
	,m_sockbuf(new sockbuf_type())
	,m_delbuf(true)
    {
      try{
	this->init(m_sockbuf);
	this->open(host,port,md);
      }OAPR_CATCH();
    }
	

    virtual ~basic_osockstream()
    {
      if(m_delbuf && m_sockbuf){
	delete m_sockbuf;
      }
    }
    virtual sockbuf_type* rdbuf()const{
      return m_sockbuf;
    }
    /**
     * Opens a socket 
     * - if ios_base::out is used, then its a client 
     * - if ios_base::in is used, there is no affect
     */
    virtual void open(const char* host,unsigned short port,int type,
		      typename sockbuf_type::sockmodes md=sockbuf_type::client);

    virtual bool is_open()const{
      return m_sockbuf->is_created()&&m_sockbuf->is_open();
    }
    virtual int set_option(Socket_Option opt, oapr_int32_t on){
      return m_sockbuf->set_option(opt,on);
    }
    virtual int get_option(Socket_Option opt, oapr_int32_t on){
      return m_sockbuf->get_option(opt,on);
    }
    virtual int close(){return m_sockbuf->close();}
    virtual int shutdown(Shutdown_How how){
      return m_sockbuf->shutdown(how);
    }
    virtual int create(int fam, int type){return m_sockbuf->create(fam,type);}
    virtual int connect(oapr_time_interval_t timeout=PR_INTERVAL_NO_TIMEOUT){
      return m_sockbuf->connect(m_si,timeout);
    }

  };
  template<typename charT, typename traits,
	   typename thread_pol_t,std::streamsize buf_sz> void
  basic_osockstream<charT,traits,thread_pol_t,buf_sz>::open(const char* host,unsigned short port,
					int type,
					typename sockbuf_type::sockmodes md)
  {
    try{
      //acceptable values; client, none
      if(md==sockbuf_type::client || md==sockbuf_type::none){
	/* verifying that the socket is not open already*/
	if(is_open()){
	  this->setstate(std::ios_base::failbit);
	  return;/**@todo check exception flag and throw*/
	}
	/* create a sockinfo and use that to create a socket*/
	/**\todo fix first 0*/
	m_si.create(host,0,port,0);
	/**@todo put exception catch and status check */
	this->create(m_si.family(),type);
	if(md==sockbuf_type::client){
	  this->connect();
	}
      }else{
	this->setstate(std::ios_base::failbit);
      }
    }OAPR_CATCH();
  }
};
#endif
