/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
#include "oapr/buildconfig.hpp"
#include "oapr/pipe.hpp"
#include "oapr/exception.hpp"
#include "oapr/exc_utils.hpp"

FILE_VERSION("$Revision: 1.3 $",oapr_io_pipe_cpp);
namespace oapr
{
  oapr_status_t basic_pipe::open(oapr_int32_t flags, oapr_int32_t perms)
  {
    if(is_open()){
      OAPR_THROW(logic_error,"pipe is already open");
    }
    oapr_status_t rv = PR_CreatePipe(&m_readp,&m_writep);

    if(rv==OAPR_SUCCESS){
      m_type=pipe;
      m_isOpen=true;
    }
    return rv;
  }
  oapr_status_t basic_pipe::open(const std::string &name,
			oapr_int32_t flags,
			oapr_int32_t perms)
  {
    if(is_open()){
      OAPR_THROW(logic_error,"pipe is already opened");
    }
    /*
    oapr_status_t rv;
    if(flags&APR_CREATE){
      OAPR_CHECK(apr_file_namedpipe_create(name.c_str(),perms,get_pool()->getp()));
    }
    rv=apr_file_open(&m_file,name.c_str(),flags,perms,get_pool()->getp());
    if(rv==APR_SUCCESS){
      m_type=named;
      m_isOpen=true;
    }
    return rv;
    */
    return OAPR_NOT_IMPLEMENTED;
  }
  oapr_status_t basic_pipe::close()
  {
    if(m_type==pipe){
      OAPR_CHECK(PR_Close(m_readp));
      OAPR_CHECK(PR_Close(m_writep));
      return OAPR_SUCCESS;
    }else{
      return basic_file::close();
    }
  }
  oapr_int32_t basic_pipe::write(const void* buff,oapr_int32_t toWrite)
  {
    if(m_type==pipe){
      return PR_Write(m_writep,buff,toWrite);
    }else{
      return basic_file::write(buff,toWrite);
    }
  }
  oapr_int32_t basic_pipe::read( void* buff,oapr_int32_t toRead)const
  {
    if(m_type==pipe){
      return PR_Read(m_readp,buff,toRead);
    }else{
      return basic_file::read(buff,toRead);
    }
  }

  oapr_status_t basic_pipe::flush()
  {
    if(m_type==pipe){
      return PR_Sync(m_writep);
    }else{
      return basic_file::flush();
    }
  }
};
