/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
#ifndef OAPR_IO_PIPE_HPP
#define OAPR_IO_PIPE_HPP
#include "oapr/oapr.hpp"
#include "oapr/buildconfig.hpp"
#include "oapr/file.hpp"

FILE_VERSION("$Revision: 1.3 $",oapr_io_pipe_hpp);

namespace oapr
{
  class basic_pipe : public basic_file
  {
  public:
    enum pipe_type{none=0,pipe=1,named=2};
  public:
    basic_pipe()
      : basic_file(),
	m_readp(0),
	m_writep(0){}
    basic_pipe(const basic_pipe& p)
      : basic_file(p),
	m_readp(p.m_readp),
	m_writep(p.m_writep),
	m_type(p.m_type){}
    virtual ~basic_pipe(){}

    virtual oapr_status_t open(oapr_int32_t flags, oapr_int32_t perms);
    virtual oapr_status_t open(const std::string &name,
		      oapr_int32_t flags,
		      oapr_int32_t perms);
    virtual oapr_int32_t write(const void* buff, oapr_int32_t toWrite);
    virtual oapr_int32_t read( void* buff,oapr_int32_t toRead)const;

    virtual oapr_status_t flush();
    virtual oapr_status_t close();

  protected:
    oapr_file_t *m_readp;
    oapr_file_t *m_writep;
    pipe_type m_type;
  };
};
#endif
