/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

#ifndef OAPR_PROCESS_HPP
#define OAPR_PROCESS_HPP

#include "oapr/oapr.hpp"
#include "oapr/buildconfig.hpp"
#include "oapr/exception.hpp"
#include "oapr/file.hpp"
#include <string>
#include <vector>
#include <assert.h>
#include "boost/shared_ptr.hpp"
#include "boost/scoped_array.hpp"
#include <prproces.h>
FILE_VERSION("$Revision: 1.3 $",process_hpp);

namespace oapr {

  class process
  {
  };

  class LIBOAPR_API child_process: public process
  {
  public:
    typedef std::vector<std::string> args_cont;
    typedef std::vector<std::string> env_cont;
  public:
    /**
     * Create process object.
     */
    child_process()
      : m_proc(0),
	m_assigned(false)
    {
    }
    /**
     * Create and start process object.
     * @param progname The program to run
     * @param args the arguments to pass to the new program.  The first
     *             one should be the program name.
     * @param env The new environment table for the new process.  This
     *            should be a list of NULL-terminated strings. This argument
     *            is ignored for APR_PROGRAM_ENV and APR_PROGRAM_PATH types
     *            of commands.
     */
    child_process(const std::string& progname, args_cont& args, const env_cont& env);
    ~child_process();

    // Local functions
    //static void fork(process **process);

    inline oapr_status_t kill(int sig) { 
      return PR_KillProcess(m_proc);
    }

    /**
     * Wait for a child process to die
     * @param exitcode The returned exit status of the child, if a child process 
     *                 dies, or the signal that caused the child to die.
     *                 On platforms that don't support obtaining this information, 
     *                 throws a not_implemented error.
     * Parameter is optional and may be NULL.     
     */
    oapr_status_t waitfor(oapr_int32_t *exitcode = NULL);

  protected:
    void create_proc(const std::string& progname, args_cont& args, const env_cont& env);
    void setup_argenv(const std::string& progname, args_cont& args, const env_cont& env);
  private:
    child_process(const child_process &cp){};
  private:
    PRProcess *m_proc;
    bool m_assigned;
    std::string m_progname;
    boost::scoped_array<char*> m_args;
    boost::scoped_array<char*> m_envp;
  };
};

#endif
