/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#include "oapr-config.h"
#include "oapr/error.hpp"
#include "oapr/buildconfig.hpp"
#include "boost/scoped_ptr.hpp"
#include "oapr/socketinfo.hpp"
#include "oapr/exception.hpp"
#include <prnetdb.h>

FILE_VERSION("$Revision: 1.4 $",socketinfo_cpp);

namespace oapr
{
  sockinfo::sockinfo()
  {

  }
  sockinfo::sockinfo(const std::string &host, oapr_int32_t fam,
		     oapr_port_t port, oapr_int32_t flags)
  {
    create(host,fam,port,flags);
  }
  sockinfo::sockinfo(const sockinfo& si)
    : m_addrt(si.m_addrt),
      m_hostname(si.m_hostname)
  {
  }
  void sockinfo::create(const std::string& hostname,
			oapr_int32_t family,
			oapr_port_t port,
			oapr_int32_t flags)
  {
    char buf[PR_NETDB_BUF_SIZE];
    PRHostEnt he;

    OAPR_CHECK(PR_InitializeNetAddr(PR_IpAddrAny,port,&m_addrt));
    OAPR_CHECK(PR_GetHostByName(hostname.c_str(),buf,sizeof(buf),&he));

    oapr_status_t rv = PR_EnumerateHostEnt(0,&he,port,&m_addrt);
    if(rv == OAPR_FAILURE){
      OAPR_THROW(runtime_error,"failed to get addr from HostEnt");
    }
    if(he.h_name){
      m_hostname = he.h_name;
    }else{
      m_hostname = hostname;
    }
  }
  std::string sockinfo::hostname()const{
    return m_hostname;
  }
  std::string sockinfo::service()const
  {
    OAPR_THROW(not_implemented,"Getting service name for socket is not implemented yet");
    std::string rv;

    return rv;
  }
  oapr_port_t sockinfo::port()const
  {
    /*
    if(!m_addrt){
      OAPR_THROW(invalid_state,"addr is not init, call create first");
    }
    */
    return PR_ntohs(PR_NetAddrInetPort(&m_addrt));
  }
  oapr_status_t sockinfo::port(oapr_port_t p)
  {
    /*
    if(!m_addrt){
      OAPR_THROW(invalid_state,"addr is not init, call create first");
    }
    */
    //    return apr_sockaddr_port_set(m_addrt,p);
    return OAPR_FAILURE;
  }

  int sockinfo::family()const
  {
    /*
    if(!m_addrt){
      OAPR_THROW(invalid_state,"addr is not init, call create first");
    }
    */
    return PR_NetAddrFamily(&m_addrt);
  }

    std::string sockinfo::ip()const
    {
      char tmpip[256]; //note this value is used in the gethost.c test
      std::string rv;
      PR_NetAddrToString(&m_addrt,tmpip,256);
      rv=tmpip;
      /* note: intentionally not freeing tmpip
       * it is alloc'd by apr on the m_addrt pool
       * so when the pool is freed, so is tmpip
       */
      return rv;
    }
}
