/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#include <iostream>
#include "oapr/app.hpp"
#include "oapr/exception.hpp"
#include "oapr/exc_utils.hpp"
#include "oapr/socket.hpp"
#include "oapr/socketinfo.hpp"

using namespace oapr;

int main(int argc,char*argv[])
{
  oapr::app app("test client");

  char sdata[]="ping";
  char rdata[sizeof(sdata)+1];
  std::string serveraddr("127.0.0.1");
  std::memset(rdata,0,sizeof(rdata));

  sockinfo si;
  basic_socket sock;
  try{
    si.create(serveraddr,0,1234,0);

    //create the socket
    OAPR_CHECK_MSG(sock.create(si.family(),SOCK_STREAM), "failed to create socket");

    //connect to the remote host
    OAPR_CHECK_MSG(sock.connect(si,PR_INTERVAL_NO_TIMEOUT), "failed to connect");

    oapr_status_t stat;
    do{
      stat=sock.write(sdata,sizeof(sdata));
      std::cerr<<"w: "<<sdata<<"\n";
      stat=sock.read(rdata,sizeof(rdata));
      std::cerr<<"r: "<<rdata<<"\n";
      std::memset(rdata,0,sizeof(rdata));
    }while(stat==OAPR_SUCCESS);
  }catch(const oapr_exception &e){
    print_stack(&std::cerr,e);
  }catch(const exception_base &e){
    print_stack(&std::cerr,e);
  }catch(const std::exception &e){
  }catch(...){
  }
}
