/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
#include <iostream>
#include "oapr/app.hpp"
#include "oapr/exception.hpp"
#include "oapr/exc_utils.hpp"
#include "oapr/ifilestream.hpp"
#include "oapr/ofilestream.hpp"
#include "oapr/iofilestream.hpp"

std::string nm("/tmp/oapr-test-filestrm");

void test_out()
{
  try{
    oapr::basic_ofilestream<char,std::char_traits<char> > ofile;
    ofile.open(nm.c_str(),std::ios_base::out,00666);
    ofile<<"line one"<<std::endl;
    ofile<<"line two"<<std::endl;
    ofile.close();
  }OAPR_CATCH()
}

void test_in()
{
  try{
    std::string data;

    oapr::basic_ifilestream<char,std::char_traits<char> > ifile;

    ifile.open(nm.c_str(),std::ios_base::in,00666);
    std::getline(ifile,data);
    std::cout<<"one: "<<data<<std::endl;
    std::getline(ifile,data);
    std::cout<<"two: "<<data<<std::endl;
    ifile.close();
  }OAPR_CATCH()
}

void test_inout()
{
  try{
    std::string data;
    oapr::basic_iofilestream<char,std::char_traits<char> > iofile;
    iofile.open(nm.c_str(),std::ios_base::out|std::ios_base::in,00666);
    iofile<<"line one"<<std::endl;
    iofile<<"line two"<<std::endl;

    iofile.seekg(0,std::ios_base::beg);
    std::getline(iofile,data);
    std::cout<<"one: "<<data<<std::endl;
    std::getline(iofile,data);
    std::cout<<"two: "<<data<<std::endl;
    iofile.close();
  }OAPR_CATCH()
}

int main(int argc, char*argv[])
{
  try{
    oapr::app app("test filestream");

    test_out();
    test_in();
    //    test_inout();  
  }catch(const oapr::oapr_exception &e){
    print_stack(&std::cerr,e);
    return 1;
  }catch(const oapr::exception_base &e){
    print_stack(&std::cerr,e);
    return 1;
  }catch(const std::exception &e){
    return 1;
  }catch(...){
    return 1;
  }
  return 0;
}
