/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
#include <iostream>
#include "oapr/oapr.hpp"
#include "oapr/app.hpp"
#include "oapr/exception.hpp"
#include "oapr/exc_utils.hpp"
#include "oapr/pipe.hpp"

#define PIPE_NAME "/tmp/oapr-pipe"

using oapr::oapr_status_t;

oapr_status_t create_pipe()
{
  try{
    oapr::basic_pipe p;
    return p.open(0,0);

  }OAPR_CATCH();
}
oapr_status_t create_named_pipe()
{
  try{
    oapr::basic_pipe np;
    /*
    OAPR_CHECK(np.open(PIPE_NAME, PR_RDWR,
		       APR_UREAD|APR_UWRITE|APR_GREAD|APR_GWRITE));
    */
    return oapr::OAPR_SUCCESS;
  }OAPR_CATCH();
}
oapr_status_t read_write()
{
  try{
    char sdata[]="testing one, two, three";
    char rdata[sizeof(sdata)+1];
    oapr::oapr_size_t written(0),read(0);

    oapr::basic_pipe p;

    std::memset(rdata,0,sizeof(rdata)+1);

    OAPR_CHECK(p.open(0,0));
    written=p.write(sdata,sizeof(sdata));
    if(written!=sizeof(sdata)){
      OAPR_THROW(oapr::runtime_error,"data written is not equal to data sent");
    }
    std::cout<<"w: "<<sdata<<std::endl;
    read=p.read(rdata,sizeof(rdata)-1);
    std::cout<<"r: "<<rdata<<std::endl;

    if(read!=written){
      OAPR_THROW(oapr::runtime_error,"read is not equal to written");
    }
    return oapr::OAPR_SUCCESS;

  }OAPR_CATCH();
}

int main(int argc, char*argv[])
{
  try{
    oapr::app app("pipe test");

    OAPR_CHECK(create_pipe());
    OAPR_CHECK(read_write());

    OAPR_CHECK(create_named_pipe());

  }OAPR_PRINT(&std::cerr,return 1);
}
