/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE   cout << "Test" << endl;
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */

#include "oapr/app.hpp"
#include "oapr/exception.hpp"
#include "oapr/exc_utils.hpp"
#include "oapr/thread.hpp"
#include "oapr/sleep.hpp"
#include <iostream>

using namespace oapr;
using namespace std;

class test_class: public thread_class
{
  public:
    test_class(): thread_class() {}
  protected:
    oapr_status_t execute();
  private:
    // My thread data
};

oapr_status_t test_class::execute()
{

  cout << "Test" << endl;
  oapr::sleep(1);//<-- doing a cheezy sleep to get main thread time to sync up
  cout << "Test2" << endl;
  oapr::sleep(1);//<-- doing a cheezy sleep to get main thread time to sync up
  cout << "Test3" << endl;
  oapr::sleep(1);//<-- doing a cheezy sleep to get main thread time to sync up
  return OAPR_SUCCESS;
}

int main(int argc,char*argv[])
{
  app app("thread test");
  cerr << "Threading test. Starting 3 threads..." << endl;
  try
  {
    {
      thread<test_class> t1(true);
      for(int i=0; 
	  i<100 && 
	    t1.state() != oapr::tsRunning && 
	    t1.state() != oapr::tsTerminated; 
	  ++i){
	cerr << "waiting for running state\r";
	oapr::sleep(1);
      }
      if(t1.state() != oapr::tsRunning && t1.state() != oapr::tsTerminated){
	OAPR_THROW(oapr::runtime_error,"failed to get a valid state from thread");
      }
      cerr << "Before join and thread free." << endl;
      cerr<<"joining t1\n";
      t1.join();
    }
    cout << "Got past freeing of threads." << endl;

  }catch(const exception_base &e){
    print_stack(&std::cerr,e);
    return 1;
  }catch(const std::exception &e){
    return 1;
  }catch(...){
    return 1;
  }
  return 0;
}
