/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#ifndef OAPR_LOGSTREAM_HPP
#define OAPR_LOGSTREAM_HPP
#include <ostream>
#include "oapu/oapu.hpp"
#include "oapu/logmessage.hpp"
#ifndef OAPR_FUNCTION
#ifdef WIN32
#define OAPR_FUNCTION __FUNCTION__
#else
#define OAPR_FUNCTION __PRETTY_FUNCTION__
#endif
#endif
namespace oapr
{

  class log_info
  {
  public:
    static void level(Log_Level lvl){log_info::m_level=lvl;}
    static Log_Level level(){return log_info::m_level;}
    static void stream(std::ostream* os){log_info::m_stream=os;}
    static std::ostream& stream(){return *log_info::m_stream;}
  private:
    static Log_Level m_level;
    static std::ostream* m_stream;
  };
};
#define OAPR_STRM oapr::log_info::stream()
#define OAPR_LOG(msg,lvl) \
oapr::log_message(__LINE__, \
	    OAPR_FUNCTION, \
	    __FILE__, \
	    msg, \
	    lvl);

template<typename char_t,typename traits_t>
std::basic_ostream<char_t,traits_t>& 
operator<<(std::basic_ostream<char_t,traits_t> &os,
	   const oapr::log_message& lm)
{
  if(lm.level()<=oapr::log_info::level()){
    os<<lm.file()<<":"<<lm.line()<<"\n\t "
      <<lm.what();
  }
  return os;
}

#endif
