/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#include "oapu/uri.hpp"

class uri_cout : public oapr::uri
{
protected:
  void on_query(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"query: "<<tmp<<std::endl;
  }
  void on_fragment(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"fragment: "<<tmp<<std::endl;
  }
  void on_param(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"param: "<<tmp<<std::endl;
  }
  void on_segment(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"segment: "<<tmp<<std::endl;
  }
  void on_path_segments(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"segments: "<<tmp<<std::endl;
  }
  void on_abs_path(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"abs_path: "<<tmp<<std::endl;
  }
  void on_path(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"path: "<<tmp<<std::endl;
  }
  void on_port(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"port: "<<tmp<<std::endl;
  }
  void on_ipv4(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"ipv4: "<<tmp<<std::endl;
  }
  void on_toplabel(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"toplabel: "<<tmp<<std::endl;
  }
  void on_domainlabel(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"domainlabel: "<<tmp<<std::endl;
  }
  void on_hostname(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"hostname: "<<tmp<<std::endl;
  }
  void on_host(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"host: "<<tmp<<std::endl;
  }
  void on_hostport(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"hostport: "<<tmp<<std::endl;
  }
  void on_userinfo(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"userinfo: "<<tmp<<std::endl;
  }
  void on_server(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"server: "<<tmp<<std::endl;
  }
  void on_reg_name(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"reg_name: "<<tmp<<std::endl;
  }
  void on_authority(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"authority: "<<tmp<<std::endl;
  }
  void on_scheme(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"scheme: "<<tmp<<std::endl;
  }
  void on_rel_segment(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"segment: "<<tmp<<std::endl;
  }
  void on_rel_path(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"rel_path: "<<tmp<<std::endl;
  }
  void on_net_path(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"net_path: "<<tmp<<std::endl;
  }
  void on_opaque_part(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"opaque_part: "<<tmp<<std::endl;
  }
  void on_hier_part(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"hier_part: "<<tmp<<std::endl;
  }
  void on_relative_uri(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"relative_uri: "<<tmp<<std::endl;
  }
  void on_absolute_uri(char const* first,char const* last)
  {
    std::string tmp(first,last);
    std::cout<<"abs_uri: "<<tmp<<std::endl;
  }
};


int main(int argc,char*argv[])
{
  std::string one("http://user@example.com:8000/path/file.html"),
    two("mailto:user@example.com"),
    three("xmpp:ralphm@ik.nu/my/resource;http:%2F%2Fjabber.org%2Fprotocol%2Fpubsub"),
    four("mysql://user@example.com:5432/dbname");
  uri_cout uric;
  std::cout<<"parsing uri: "<<one<<std::endl;
  uric.parse(one);

  std::cout<<"parsing uri: "<<two<<std::endl;
  uric.parse(three);

  std::cout<<"parsing uri: "<<three<<std::endl;
  uric.parse(three);

  std::cout<<"parsing uri: "<<four<<std::endl;
  uric.parse(four);
}
