/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

#include "oapx/csv.hpp"
#include <string>
#include <vector>
#include "oapx/strtools.hpp"

using namespace std;

namespace oapr {

void tokenize_strip_quotes(const string& str, vector<string>& tokens, const string& delimiters)
{
  string delim(delimiters + "\"");
  /**\todo Doesn't handle commas (or quotes) inside quotes */
  tokens.clear();
  
  string::size_type startpos = 0;
  string::size_type cutpos = 0;
  string::size_type pos = str.find_first_of(delim, 0);
  
  bool in_quotes = false;

  while (pos != string::npos && startpos != string::npos)
  {
    char c = str[pos];
    if(in_quotes)
    {
      if(c == '"') in_quotes = false;
      startpos = pos+1;
    } else {
      if(c == '"') {
        in_quotes = true;
        startpos = pos+1;
      } else {
        tokens.push_back(strip(str.substr(cutpos, pos - cutpos)));
        cutpos = pos+1;
        startpos = cutpos;
      }
    }
    pos = str.find_first_of(delim, startpos);
    if(pos == string::npos && !in_quotes) tokens.push_back(strip(str.substr(cutpos, str.length()-cutpos)));
  }
}

}

/*void tokenize_strip_quotes(const string& str, vector<string>& tokens, const string& delimiters)
{
  tokens.clear();
  
  // -> no thanks // Skip delimiters at beginning.
  string::size_type lastpos = str.begin(); // find_first_not_of(delimiters, 0);
  // Find first "non-delimiter".
  string::size_type pos = str.find_first_of(delimiters, lastpos);
  
//  cout << "tokenize_strip_quotes(" << str << ", tokens, " << delimiters << ") pos: " << pos << " lastpos: " << lastpos << endl;
  
  while (pos != string::npos || lastpos != string::npos)
  {
    // Found a token, add it to the vector.
    tokens.push_back(strip(str.substr(lastpos, pos - lastpos)));
//    cout << "tok: " << tokens[tokens.size()-1] << endl;
    // Skip delimiters.  Note the "not_of"
    lastpos = str.find_first_not_of(delimiters, pos);
    // Find next "non-delimiter"
    pos = str.find_first_of(delimiters, lastpos);
  }
}

*/



