/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

#ifndef OAPR_CSV_HPP
#define OAPR_CSV_HPP

#include <string>
#include <vector>
#include <iostream>
#include "oapr/ifilestream.hpp"
#include "oapr/file.hpp"
#include "oapx/strtools.hpp"

namespace oapr
{
  void tokenize_strip_quotes(const std::string& str, std::vector<std::string>& tokens, const std::string& delimiters = " ");

  class adapter {
    public:
      adapter() {}
      virtual ~adapter() {}
  };
  
  template<class istream_class>
  class csv_adapter: public adapter
  {
    public:
      explicit csv_adapter(istream_class& istream):
        m_stream(istream),
        m_strip_quotes(true) {}
        
/*      csv_adapter(const csv_adapter& src):
        m_stream(src.m_stream),
        m_fields(src.m_fields),
        m_line(src.m_line),
        m_strip_quotes(src.m_strip_quotes) {} */
        
      virtual ~csv_adapter() {}
      
      bool readline()
      {
        std::string line;
        std::vector<std::string> fields;
        std::getline(m_stream, line);
        if(m_strip_quotes) tokenize_strip_quotes(line, m_fields, ","); else tokenize(line, m_fields, ",");
//        cout << "(debug) line: " << line << " fields_count: " << m_fields.size() << endl;
        return !m_stream.eof();
      }
      
      std::vector<std::string>& fields() {return m_fields;}
      
      bool strip_quotes() { return m_strip_quotes; }
      void strip_quotes(bool value) { m_strip_quotes = value; }
      
    private:
      csv_adapter(const csv_adapter& src) {}
      istream_class& m_stream;
      std::vector<std::string> m_fields;
      bool m_strip_quotes;
  };
  
  typedef basic_ifilestream<char,std::char_traits<char> > basic_file_stream_type;
  class csv_ifilestream: public basic_file_stream_type, 
			 public csv_adapter<basic_file_stream_type>
  {
  public:
    typedef pool_mgr::pool_t pool_t;
    typedef pool_mgr::pool_ptr pool_ptr;
  public:
    csv_ifilestream(const pool_ptr& p)
      : basic_file_stream_type(p),
        csv_adapter<basic_file_stream_type>(*(basic_file_stream_type*)this) {}
      
      csv_ifilestream(const pool_ptr&  p,const char* nm,
		      std::ios_base::openmode md = std::ios::in,
		      apr_int32_t perms=APR_OS_DEFAULT)
      : basic_file_stream_type(p, nm, md, perms),
        csv_adapter<basic_file_stream_type>(*(basic_file_stream_type*)this) {}
                
      virtual ~csv_ifilestream() {}
    private:
//      csv_ifilestream(const csv_ifilestream& src) {}
  };

};

#endif
